require "simb/version"
require "github_api"
require "fileutils"

module Simb
  ROOTDIR="#{ENV['HOME']}/.simb/"

  def self.editor file
    system("#{ENV['EDITOR']} #{file}")
  end

  def self.add_header file, post_name
    File.open(file, 'a') do |f|
      f.puts  "---"
      f.puts "layout: post"
      f.puts "title: #{post_name}"
      f.puts "date: #{Time.now}"
      f.puts "categories:"
      f.puts "---"
    end
  end

  def self.repo_exists? (user, repo)
    repos = Github::Client::Repos.new
    not repos.list(user: user).select{|e| e.name == repo}.first.nil?
  end

  def self.post_filename(post_name)
    return "#{Time.now.strftime("%F")}-#{post_name.gsub(" ", "-")}.markdown"
  end

  def self.init_blog(name)
    `jekyll new #{ROOTDIR}#{name}`
  end

  def self.new_post(post_name, blog)
    file = "#{ROOTDIR}#{blog}/_posts/#{post_filename post_name}"
    if File.exists? file
      editor file
      return
    end
    `touch #{file}`
    add_header file, post_name
    editor file
  end

  def self.list_posts(blog)
    Dir.chdir("#{ROOTDIR}#{blog}/_posts") do
      Dir.glob("*").map do |a|
        if a.split('-').size < 4
          [a.gsub('.markdown', ''), a]
        else
          f = "[#{a.split('-')[0..2].join('-')}]"
          f += " #{a.split('-')[3..-1].join(' ').gsub('.markdown', '')}"
          [f, a]
        end
      end
    end
  end

  def self.list_blogs
    Dir.chdir("#{ROOTDIR}") do
      Dir.glob("*/").map{|a|a[0..-2]}.join("\n")
    end
  end


  def self.edit_config(blog)
    Dir.chdir("#{ROOTDIR}#{blog}") do
      editor "_config.yml"
    end
  end

  def self.blog_initted?(blog) # is that even a word? pffffttt...
    Dir.exists?("#{ROOTDIR}#{blog}/.git/")
  end

  def self.commit(blog)
    Dir.chdir("#{ROOTDIR}#{blog}") do
      `git pull origin gh-pages` # do any merges
      `git add --all && git commit -m "new posts (#{Time.now})"`
      `git push --force origin gh-pages`
    end
  end

  def self.create_repo(blog, user, pass)
    github = Github.new basic_auth: "#{user}:#{pass}"
    if repo_exists? user, blog
      if (ask "A repo named '#{blog}' already exists! Commit? (y/n)") !~ /^[yY].*/
        abort "Aborting!"
      end
    else
      github.repos.create name: "#{blog}",
        description: "Blog for #{blog}",
        private: false,
        has_issues: false,
        has_wiki: false,
        has_downloads: false
    end
  end

  def self.config_remote(blog, user)
    Dir.chdir("#{ROOTDIR}#{blog}") do
      `git init .`
      `git remote add origin git@github.com:#{user}/#{blog}.git`
      `git checkout -b gh-pages`
      `git pull` # get code if repo already exists
      commit(blog) # go back and publish now
    end
  end

  def self.add_file(file, blog)
    FileUtils.copy(file, "#{ROOTDIR}/#{blog}/_posts/#{post_filename file.gsub(/\..*/, '')}")
  end
end
